       CBL RENT,DLL,PGMN(LM),LIB,LIST,MAP
      *----------------------------------------------------------------
      *  EXAMPLE OF COBOL CALLING THE ODPP FUNCTION AGE
      *  THIS PROGRAM MUST BE COMPILED WITH THE FOLLOWING OPTIONS:
      *     RENT,DLL,PGMN(LM),LIB
      *----------------------------------------------------------------
       ID DIVISION.
       PROGRAM-ID. "ODPPAGE".
       ENVIRONMENT DIVISION.
       INPUT-OUTPUT SECTION.
       FILE-CONTROL.
           SELECT INPUT-FILE
           ASSIGN TO INDD
           ORGANIZATION IS SEQUENTIAL
           ACCESS IS SEQUENTIAL.
       DATA DIVISION.
       FILE SECTION.
       FD  INPUT-FILE
           RECORDING MODE IS F.
       01  INPUT-RECORD PIC X(10).
      *----------------------------------------------------------------
       WORKING-STORAGE SECTION.
      *----------------------------------------------------------------

      *
      * The following copybook contains ODPP structure definitions
      * and constants
      *
       COPY ODPPWAPI.

       01  INPUT-DONE PIC X VALUE 'N'.
       01  INPUT-DATE PIC X(10).
       01  OUTPUT-DATE PIC X(20).
       01  SVC-TOKEN       PIC S9(9) COMP-5.

       01 COLUMN-NAME PIC X(30).
       77 NULL-PTR POINTER VALUE NULL.

      *
      * Provider name and country definition (which determines
      * rules for national id format)
      *
       77 PRV-NAME PIC X(4) VALUE Z"AGE".
       77 SOURCE-DATE-FORMAT-VAL PIC X(14) VALUE Z"%YYYY/%MM/%DD".
       01 SOURCE-DATE-FORMAT.
          05  DATE-FORMAT-DBMS-TYPE PIC X VALUE 'N'.
          05  FILLER PIC X(3).
          05  DATE-FORMAT-CODE-PAGE PIC S9(9) COMP-5 VALUE -1.
          05  DATE-FORMAT-PTR POINTER.

       77 CM-ARG-PTR POINTER.
       77 ALLOC-INIT-BLOCK PIC 9 BINARY VALUE 1.

       PROCEDURE DIVISION.
           MOVE 0 TO RETURN-CODE

           OPEN INPUT INPUT-FILE.
           PERFORM MASK-INIT.
           PERFORM MASK-SERVICE
             WITH TEST BEFORE UNTIL INPUT-DONE EQUAL 'Y'.
           PERFORM MASK-TERM.
           CLOSE INPUT-FILE.
           GOBACK.

       MASK-INIT.
      *
      * Initialize the ODPP framework
      *
           SET CM-ARG-PTR TO NULL.
           CALL 'Provider_FrmwInit'
             USING
               BY VALUE CM-ARG-PTR
             RETURNING RETURN-CODE.
           IF RETURN-CODE = ODPP-OS-LIB-MUTEX-ALREADY-INIT THEN
             MOVE 0 TO RETURN-CODE
           ELSE
             IF RETURN-CODE NOT = 0 THEN
               DISPLAY
                 "Framework initialization failed rc=" RETURN-CODE
               GOBACK
             END-IF
           END-IF.

      *
      * Initialize the provider
      *
           MOVE LOW-VALUES TO DP-SVC-DEF-BODY.
           MOVE LOW-VALUES TO DP-FIELD-DEF-BODY(1).
           MOVE LOW-VALUES TO DP-INIT-OP-DEF-BODY(1),
                              DP-INIT-OP-DEF-BODY(2),
                              DP-INIT-OP-DEF-BODY(3).

           MOVE LENGTH OF DP-SVC-DEF TO STRUCT-LEN OF DP-SVC-DEF.
           COMPUTE STRUCT-LEN OF DP-FIELD-DEF-ARRAY(1)
             = LENGTH OF DP-FIELD-DEF-HEADER
               + LENGTH OF DP-FIELD-DEF-BODY.
           COMPUTE STRUCT-LEN OF DP-INIT-OP-DEF-ARRAY(1),
                   STRUCT-LEN OF DP-INIT-OP-DEF-ARRAY(2),
                   STRUCT-LEN OF DP-INIT-OP-DEF-ARRAY(3)
             = LENGTH OF DP-INIT-OP-DEF-HEADER
               + LENGTH OF DP-INIT-OP-DEF-BODY.

           SET PARAMS-PTR TO ADDRESS OF DP-INIT-OP-DEF-ARRAY(1).
           SET FLD-DEF-PTR TO ADDRESS OF DP-FIELD-DEF-ARRAY(1).
           MOVE X"01" TO COPY-TO-DEST.

           MOVE "M" TO SUB-TYPE(1).

           MOVE RDB-NONE TO DATA-DBMS-TYPE(1).
           MOVE -1 TO DATA-CODE-PAGE(1).
           MOVE ODPPDATATYPE-DATETIME-SZ TO DATA-TYPE(1).
           MOVE 1 TO FLD-COUNT.
           MOVE 5 TO PARAM-COUNT.

      *
      * Initialize the ODPP rowset, row and field  structures
      *
           MOVE LOW-VALUES TO DP-ROWSET-DEF-BODY.
           SET NEXT-PTR OF DP-ROW-DEF TO NULL.
           MOVE LENGTH OF DP-ROWSET-DEF
             TO STRUCT-LEN OF DP-ROWSET-DEF.
           MOVE 1 TO ROW-COUNT OF DP-ROWSET-DEF.

           MOVE LOW-VALUES TO DP-ROW-DEF.
           MOVE LENGTH OF DP-ROW-DEF
             TO STRUCT-LEN OF DP-ROW-DEF.

           MOVE LOW-VALUES TO DP-FIELD-DATA-DEF-BODY(1).
           SET NEXT-PTR OF DP-FIELD-DATA-DEF(1) TO NULL.
           MOVE LENGTH OF DP-FIELD-DATA-DEF
             TO STRUCT-LEN OF DP-FIELD-DATA-DEF(1).

           SET NEXT-PTR OF DP-INIT-OP-DEF-ARRAY(1) TO
             ADDRESS OF DP-INIT-OP-DEF-ARRAY(2).
           SET NEXT-PTR OF DP-INIT-OP-DEF-ARRAY(2) TO
             ADDRESS OF DP-INIT-OP-DEF-ARRAY(3).
           SET NEXT-PTR OF DP-INIT-OP-DEF-ARRAY(3) TO
             ADDRESS OF DP-INIT-OP-DEF-ARRAY(4).
           SET NEXT-PTR OF DP-INIT-OP-DEF-ARRAY(4) TO
             ADDRESS OF DP-INIT-OP-DEF-ARRAY(5).

           MOVE ODPP-OPR-SOURCE-DATE-FORMAT TO PARAMETER-ID(1).
           MOVE PARAM-VAL-MC TO VALUE-SUB-TYPE(1).
           SET DATE-FORMAT-PTR TO ADDRESS OF SOURCE-DATE-FORMAT-VAL.
           SET PARAM-VAL-PTR(1) TO ADDRESS OF SOURCE-DATE-FORMAT.
           MOVE 13 TO VALUE-BUF-BYTES(1).

           MOVE ODPP-OPR-SOURCE-COLINDEX TO PARAMETER-ID(2).
           MOVE PARAM-VAL-NUM TO VALUE-SUB-TYPE(2).
           MOVE 0 TO PARAM-VAL-INT(2).
           MOVE ODPP-OPR-METHOD TO PARAMETER-ID(3).
           MOVE PARAM-VAL-NUM TO VALUE-SUB-TYPE(3).
           MOVE ODPP-METHOD-MASK TO PARAM-VAL-INT(3).

           MOVE ODPP-OPR-SWITCH-NA TO PARAMETER-ID(4).
           MOVE PARAM-VAL-NONE TO VALUE-SUB-TYPE(4).

      *
      * Age the date by 5 years
      *
           MOVE ODPP-OPR-AGE-YEAR TO PARAMETER-ID(5).
           MOVE PARAM-VAL-NUM TO VALUE-SUB-TYPE(5).
           MOVE 5 TO PARAM-VAL-INT(5).

           CALL 'Provider_Init'
             USING
               BY VALUE ADDRESS OF SVC-TOKEN,
               BY VALUE ADDRESS OF PRV-NAME,
               BY VALUE 3,
               BY VALUE ADDRESS OF DP-SVC-DEF,
               BY VALUE ALLOC-INIT-BLOCK
             RETURNING RETURN-CODE.

           IF RETURN-CODE NOT = 0 THEN
             DISPLAY
               "Provider initialization failed rc=" RETURN-CODE
             GOBACK
           END-IF.

       MASK-SERVICE.
           READ INPUT-FILE INTO INPUT-DATE
             AT END MOVE 'Y' TO INPUT-DONE.
           IF INPUT-DONE EQUAL 'Y' THEN
             GOBACK
           END-IF.

      *
      * Populate the rowset structure
      *
           SET ROW-DEFINE-PTR TO ADDRESS OF DP-ROW-DEF.
           MOVE 0 TO LAST-PRCS-ROW.
           MOVE X"00" TO PROCESSED.
           MOVE X"00" TO HAS-ERROR.
           MOVE X"00" TO SRC-NULL(1).

      *
      * Set up the data definition structure
      *
           COMPUTE SRC-BUF-LEN(1) = FUNCTION LENGTH(INPUT-DATE).
           SET SRC-BUF-PTR(1) TO ADDRESS OF INPUT-DATE.
           COMPUTE DST-BUF-LEN(1) = FUNCTION LENGTH(OUTPUT-DATE).
           SET DST-BUF-PTR(1) TO ADDRESS OF OUTPUT-DATE.

      *
      * Set up the row definition structure
      *
           SET FLD-DATA-DEFINE-PTR TO
              ADDRESS OF DP-FIELD-DATA-DEF-ARRAY(1).
           MOVE 1 TO S-COUNT OF DP-ROW-DEF.
           MOVE SPACES TO OUTPUT-DATE.

      *
      * Call the provider service
      *
           CALL 'Provider_Service'
             USING
               BY VALUE SVC-TOKEN,
               BY VALUE ODPP-METHOD-DEFAULT,
               BY VALUE NULL-PTR,
               BY VALUE ADDRESS OF DP-ROWSET-DEF
             RETURNING RETURN-CODE.
           IF RETURN-CODE NOT = 0 THEN
             DISPLAY "NID service failed rc=" RETURN-CODE
             GOBACK
           END-IF.

           DISPLAY "Original date=" INPUT-DATE "  Masked date="
              OUTPUT-DATE.
           MOVE 0 TO RETURN-CODE.

       MASK-TERM.
           CALL 'Provider_Term'
             USING
               BY VALUE SVC-TOKEN
             RETURNING RETURN-CODE.
