/* 
* -cr_beg-
* ********************************************************************
* IBM Confidential
* OCO Source Materials
* 5724-U38, 5724-U39, 5724-U48, 5655-V17, 5655-V19, 5655-V35, 5655-V37
* Copyright IBM Corp. 2012,2013
* The source code for this program is not published or otherwise
* divested of its trade secrets, irrespective of what has
* been deposited with the U.S. Copyright Office.
* ********************************************************************
* -cr_end-
*/ 
/**********************************************************************/
/*                                                                    */
/* MEMBER NAME: FOPPSMEX                                              */
/*                                                                    */
/* DESCRIPTIVE NAME: Optim/z common structures for column map exits   */
/*                                                                    */
/* STATUS: Optim/z Release 7.1.0 with service                         */
/*                                                                    */
/* IBM Confidential                                                   */
/* 5655-V03, 5655-V04, 5655-V05, 5697-P39, 5655-V06                   */
/* ) Copyright IBM Corp. 2008 , 2009                                  */
/* The source code for this program is not published or otherwise     */
/* divested of its trade secrets, irrespective of what has            */
/* been deposited with the U.S. Copyright Office                      */
/*                                                                    */
/* MEMBER CATEGORY:  Field Definitions                                */
/*                                                                    */
/*                                                                    */
/**********************************************************************/
/*----------------------------------------------------------------------
 * FOPPSMEX.H - Mainframe common structures used for column map exits
 *--------------------------------------------------------------------*/
/* Change Log
 *
 * ID  Problem  Rel YYMMDD UID Description
 *-----------------------------------------------------------------------------
 * $A1=         710 111103 LAB initial version                             @A1A
 *--------------------- end of development changes ----------------------------
 *--------------------- end of service changes --------------------------------
 */

#ifndef __FOPPSMEX_H
#define __FOPPSMEX_H

#ifdef __cplusplus
extern "C" {                /* Assume C declarations for C++   */
#endif  /* __cplusplus */

#ifdef __IBMC__
#pragma pack(1)
#endif

#define UCHAR            unsigned char
#define USHORT           unsigned short
#define ULONG            unsigned long
#define CRE_ID_LEN       128
#define TBL_NAME_LEN     128
#define TRUE             1

#define   OPZ_DB2SUB_LEN       4   /* DB2 Subsystem Length           */
#define   OPZ_RELEASE_LEN      4   /* Release identified length      */
#define   OPZ_AUTH_ID_LEN      8   /* Authorization ID Length        */
#define   OPZ_CID_LEN        128   /* Creator ID Length              */
#define   OPZ_TBL_NAME_LEN   128   /* Table Name Length              */
#define   OPZ_COL_NAME_LEN    30   /* Column Name Length             */
#define   OPZ_COL_PARM_LEN   500   /* Column Map Entry Parm Length   */
#define   OPZ_CWA_MSG_LEN    252   /* CWA space used for abort msg   */

#define   DB2CHAR            452   /* Character <= 254 Bytes         */
#define   DB2VARCHAR         448   /* Var Len Character <= 254       */
#define   DB2LONGVARCHAR     456   /* Long Var Len Character > 254   */
#define   DB2INT             496   /* Integer - 32 Bits              */
#define   DB2SMALLINT        500   /* Small Integer - 16 Bits        */
#define   DB2DECIMAL         484   /* Decimal Number <= 15 Digits    */
#define   DB2FLOAT           480   /* Floating Point (Small & Large) */
#define   DB2DATE            384   /* Date - MM/DD/YYYY              */
#define   DB2TIME            388   /* Time - HH:MM:SS                */
#define   DB2TIMESTAMP       392   /* TimeStamp -                    */
#define   DB2ROWID           904   /* Rowid introduced DB2 V6        */
#define   DB2BLOB            404   /* BLOB  introduced DB2 V6        */
#define   DB2CLOB            408   /* CLOB  introduced DB2 V6        */
#define   DB2DBCLOB          412   /* DBCLOB new in    DB2 V6        */
#define   DB2BLOBLOC         960   /* BLOB  locator                  */
#define   DB2CLOBLOC         964   /* CLOB  locator                  */
#define   DB2DBCLOBLOC       968   /* DBCLOB locator                 */
#define   DB2VARG            464   /* VARGRAPHIC                     */
#define   DB2GRAPHIC         468   /* GRAPHIC <= 127 bytes           */
#define   DB2LONGVARG        472   /* LONG VARGRAPHIC                */

#define DSQLVARSUB_SRC      9999   /* Reserved value to set Available*/

#define RC_EXIT_SUCCESS   0        /* Exit formatted return value    */
#define RC_EXIT_SKIPROW   4        /* Exit requested skip row        */
#define RC_EXIT_ABORT     8        /* Exit requested abort process   */

typedef struct s_VarChar
{
    short           len;       /* length of VarChar */
    unsigned char   vchars[1]; /* character data    */
}  VARCHAR;

/*--------------------------------------------------------------*
 * Name:          COLMAP_PARMS                                  *
 * Description:                                                 *
 * Parameters that are passed to an Optim/z Column Map Exit.    *
 *--------------------------------------------------------------*/
typedef        struct sColMap_Parms {
  UCHAR             SSN[4];            /* Subsystem Name              */
  UCHAR             OptimRel[4];
  UCHAR             UserID[8];         /* Primary SQL ID              */
  UCHAR             SQLID[128];        /* Current SQL ID              */
  UCHAR             SCreatorIDV[CRE_ID_LEN];/* Source Creator ID    */
  short             STableNameL;       /* Length of source Tbl Name   */
  UCHAR             STableNameV[TBL_NAME_LEN]; /* Source Tbl Name     */
  UCHAR             DCreatorIDV[CRE_ID_LEN];   /* Dest Creator ID     */
  short             DTableNameL;       /* Length of dest Tbl Name     */
  UCHAR             DTableNameV[TBL_NAME_LEN]; /* Dest Tbl Name       */
  short             DSQLVarSub;        /* Destination SQLVAR subscript*/
  UCHAR             Flag1;
     #define EXITPARM_FLAG1_STARTEXIT  0x01
     #define EXITPARM_FLAG1_LASTCALL   0x02
     #define EXITPARM_FLAG1_LASTTABLE  0x04
  UCHAR             filler1;           // used for alignment only
  void             *pGlobalWork;       // Global work area (256 bytes)
  void             *pTblWork;          // Table work area (256 bytes)
  UCHAR            *pCallParms;        // User parms passed to exit
  long              CurrentRow;        // current row number
  long              TotRows;           // total number of rows in table
} COLMAP_PARMS;

/*--------------------------------------------------------------*
 * SQLCA in C                                                   *
 *--------------------------------------------------------------*/
typedef struct s_SQLCA
{
    UCHAR  sqlcaid[8];      /* ID */
    long   sqlcabc;         /* byte count */
    long   sqlcode;         /* return code */
    short  sqlerrml;        /* len of error msg parms */
    UCHAR  sqlerrmc[70];    /* error msg parms */
    UCHAR  sqlerrp[8];      /* impl dependent error info? */
    long   sqlerrd[6];      /* return info */
    UCHAR  sqlwarn[8];      /* warning flags */
    UCHAR  sqlext[8];       /* reserved */
} SQLCA;

/*----------------------------------------------------------------------
 *       DB2 for z/OS SQLDA Variable entry structure
 *--------------------------------------------------------------------*/
typedef struct sSQLVar {
  short DataType;                  // col type/null code
  short Length;                    // column length
  char  *pData;                    // ptr to column data
  short *pInd;                     // ptr to null ind var
  short NameLen;                   // column name length
  char  Name[OPZ_COL_NAME_LEN]; // column name / label
} SQLVAR;

/*----------------------------------------------------------------------
 *       DB2 for z/OS SQLDA structure
 *--------------------------------------------------------------------*/
typedef struct sSQLDA {
  char  ID[8];                 // ID
  long  bc;                   // byte count
  short TotCols;               // total number of var occurances
  short NumCols;               // used number of var occurances
  SQLVAR Col[1];               // individual column SQLDA
} SQLDA;

// define size of above structure
#define SQLDA_ENTRIES(n)                                      \
      (sizeof(SQLDA) + ((n - 1) * sizeof(SQLVAR)))

#ifdef __cplusplus
}                                    /* End of extern "C" { */
#endif  /* __cplusplus */

#endif  /* #ifndef __FOPPSMEX_H */
